{
   Report Workshop demo
   https://www.trichview.com

   This demo shows how to use data from TListView in reports.

   Query processor TRVReportListViewQueryProcessor is implemented in
   RVReportListViewQueryProcessor.pas, included in this demo.

   In this demo, we do not use data provider components. We create
   query processors in RVReportGenerator1.OnCreateQueryProcessor event
}

unit Unit1;

interface

uses
  // Standard units
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls,
  // TRichView units
  RVStyle, RVScroll, RichView, RVTable, RVTypes,
  // Report Workshop units
  RVReportTable, RVReportGenerator, RVReportDataProvider, RVReportTypes,
  // This demo units
  RVReportListViewQueryProcessor;


type
  TForm1 = class(TForm)
    RichView1: TRichView;
    RVStyle1: TRVStyle;
    ListView1: TListView;
    RVReportGenerator1: TRVReportGenerator;
    Button1: TButton;
    ListView2: TListView;
    procedure FormCreate(Sender: TObject);
    procedure RVReportGenerator1CreateQueryProcessor(
      Sender: TCustomRVReportGenerator; const DataQuery: TRVUnicodeString;
      var QueryProcessor: TRVReportQueryProcessor);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure CreateReportTemplate;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

const
  // indexes in RVStyle1.TextStyles[]
  TEXTSTYLE_NORMAL = 0;
  TEXTSTYLE_HEADING = 1;
  TEXTSTYLE_TABLEHEADER = 2;
  // indexes in RVStyle.ParaStyles[]
  PARASTYLE_LEFT = 0;
  PARASTYLE_CENTER = 1;
  PARASTYLE_RIGHT = 2;

procedure ClearTable(Table:  TRVTableItemInfo);
var
  r, c: Integer;
begin
 for r := 0 to Table.RowCount - 1 do
   for c := 0 to Table.ColCount - 1 do
     if Table.Cells[r, c] <> nil then
       Table.Cells[r, c].GetRVData.Clear;
end;

procedure TForm1.CreateReportTemplate;
var
  Table: TRVReportTableItemInfo;
begin
  RichView1.AddNL('Persons from ListView', TEXTSTYLE_HEADING, PARASTYLE_CENTER);

  Table := TRVReportTableItemInfo.CreateEx(2, 4, RichView1.RVData);
  Table.BorderStyle := rvtbColor;
  Table.CellBorderStyle := rvtbColor;
  Table.BorderWidth := 1;
  Table.CellBorderWidth := 1;
  Table.CellHSpacing := -1;
  Table.CellVSpacing := -1;
  Table.BorderHSpacing := 0;
  Table.BorderVSpacing := 0;
  Table.HeadingRowCount := 1;
  Table.Color := clNone;
  Table.HeadingRowColor := clRVReportGaugeYellowColor;
  Table.EvenRowsColor := clBtnFace;
  Table.CellHPadding := 5;
  ClearTable(Table);
  Table.Cells[0,0].AddNL('Person', TEXTSTYLE_TABLEHEADER, PARASTYLE_CENTER);
  Table.Cells[0,1].AddNL('Height, cm', TEXTSTYLE_TABLEHEADER, PARASTYLE_CENTER);
  Table.Cells[0,2].AddNL('Weight, kg', TEXTSTYLE_TABLEHEADER, PARASTYLE_CENTER);
  Table.Cells[0,3].AddNL('Age',    TEXTSTYLE_TABLEHEADER, PARASTYLE_CENTER);
  Table.Cells[1,0].AddNL('{1}', TEXTSTYLE_NORMAL, PARASTYLE_LEFT);
  Table.Cells[1,1].AddNL('{2}', TEXTSTYLE_NORMAL, PARASTYLE_RIGHT);
  Table.Cells[1,2].AddNL('{3}', TEXTSTYLE_NORMAL, PARASTYLE_RIGHT);
  Table.Cells[1,3].AddNL('{4}', TEXTSTYLE_NORMAL, PARASTYLE_RIGHT);
  with Table.RowGenerationRules.Add do
  begin
    FirstRow := 1;
    RowCount := 1;
    DataQuery := 'ListView1';
  end;
  RichView1.AddItem('', Table);

  RichView1.AddNL('Men''s Size Table from ListView', TEXTSTYLE_HEADING, PARASTYLE_CENTER);  

  Table := TRVReportTableItemInfo.CreateEx(2, 5, RichView1.RVData);
  Table.BorderStyle := rvtbColor;
  Table.CellBorderStyle := rvtbColor;
  Table.BorderWidth := 1;
  Table.CellBorderWidth := 1;
  Table.CellHSpacing := -1;
  Table.CellVSpacing := -1;
  Table.BorderHSpacing := 0;
  Table.BorderVSpacing := 0;
  Table.HeadingRowCount := 1;
  Table.Color := clNone;
  Table.HeadingRowColor := clRVReportGaugeGreenColor;
  Table.CellHPadding := 5;
  ClearTable(Table);
  Table.Cells[0,0].AddNL('Size',       TEXTSTYLE_TABLEHEADER, PARASTYLE_CENTER);
  Table.Cells[0,1].AddNL('Height, cm', TEXTSTYLE_TABLEHEADER, PARASTYLE_CENTER);
  Table.Cells[0,2].AddNL('Chest, cm',   TEXTSTYLE_TABLEHEADER, PARASTYLE_CENTER);
  Table.Cells[0,3].AddNL('Waist, cm',  TEXTSTYLE_TABLEHEADER, PARASTYLE_CENTER);
  Table.Cells[0,4].AddNL('Hip, cm',    TEXTSTYLE_TABLEHEADER, PARASTYLE_CENTER);
  Table.Cells[1,0].AddNL('{1}', TEXTSTYLE_NORMAL, PARASTYLE_CENTER);
  Table.Cells[1,1].AddNL('{2}', TEXTSTYLE_NORMAL, PARASTYLE_CENTER);
  Table.Cells[1,2].AddNL('{3}', TEXTSTYLE_NORMAL, PARASTYLE_CENTER);
  Table.Cells[1,3].AddNL('{4}', TEXTSTYLE_NORMAL, PARASTYLE_CENTER);
  Table.Cells[1,4].AddNL('{5}', TEXTSTYLE_NORMAL, PARASTYLE_CENTER);  
  with Table.RowGenerationRules.Add do
  begin
    FirstRow := 1;
    RowCount := 1;
    DataQuery := 'ListView2';
  end;
  RichView1.AddItem('', Table);

  RichView1.Format;


end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  CreateReportTemplate;
end;

procedure TForm1.RVReportGenerator1CreateQueryProcessor(
  Sender: TCustomRVReportGenerator; const DataQuery: TRVUnicodeString;
  var QueryProcessor: TRVReportQueryProcessor);
var
  i: Integer;
begin
  // Comparing DataQuery with names of all TListViews on this form.
  // If equal, creating TRVReportListViewQueryProcessor for them
  for i := 0 to ComponentCount - 1 do
    if (Components[i] is TListView) and
      (CompareText(DataQuery, Components[i].Name) = 0) then
      QueryProcessor := TRVReportListViewQueryProcessor.Create(TListView(Components[i]));
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  RVReportGenerator1.Execute(RichView1);
  RichView1.Format;
  Button1.Enabled := False;
end;

end.
